#!/usr/bin/env node
var helper = require("./helper.js")
const koa = require("koa");
const router = require("koa-router")();
const koaBody = require('koa-body')({ multipart: true });
const app = new koa();
var rd = require("rd");
var path = require("path")
var fs = require("fs")

var archiver = require('archiver');
var exec = require('child_process').exec;




let args = process.argv.skip(2);
console.log(args);

if (!helper.get_arg_value("-key")) {
    throw new Error("必须在参数中指定-key 此为签名校验的key");
    return;
}

if (helper.is_have_arg("-pm2")) {
    exec_cmd(process.cwd(), `pm2 start ${process.argv[1]} -- ${args.join(" ").replace("-pm2", "")}`)
    return;
}



const mch_key = helper.get_arg_value("-key");
const port = parseInt(helper.get_arg_value("--port")) || 45438;













router.all("*", koaBody)

router.all("*", async function (ctx, next) {
    ctx.request.data = { ...ctx.query, ...ctx.request.body };
    ctx.success = function (data, extend) {
        ctx.type = "application/json";
        var json = JSON.stringify({ result: true, data: data, ...extend });
        var callback = ctx.request.data.callback;
        if (callback) json = callback + "(" + json + ")";
        ctx.body = json;
    }
    ctx.error = function (errmsg, extend) {
        ctx.type = "application/json";
        var json = JSON.stringify({ result: false, message: errmsg, ...extend });
        var callback = ctx.request.data.callback;
        if (callback) json = callback + "(" + json + ")";
        ctx.body = json;
    }
    return next();
})

/**
 * 接收文件路由
 * @to 文件发布到哪里，详细的路径，带文件名和后缀
 * @files 上传上来的文件，每次只能上传一个
 * @sign 签名
 */
router.post("/receiver", async function (ctx) {
    let { files, fields } = ctx.request.body;
    let { to, sign } = fields;
    delete fields.sign;
    //标准化路径格式
    if (to) to = to.replace(new RegExp("\\\\", "g"), path.sep);
    //等待文件接收完成
    await new Promise((resolve, reject) => {
        files.file._writeStream.on("close", function () {
            resolve(true);
        })
        if (files.file._writeStream.closed == true) {
            resolve(true)
        }
    });
    //校验签名
    if (await helper.sign_receiver(fields, files.file.path, mch_key) != sign) {
        return ctx.error("错误的签名")
    }
    copyfile(files.file.path, to);
    return ctx.success("上传成功", { to: to, message: "上传成功" });
});



//备份目录
router.post("/bak", async function (ctx) {
    let { files, fields } = ctx.request.body;
    let { bak, dir, sign } = fields;
    delete fields.sign;
    //标准化路径
    bak = helper.normal_path(bak);
    dir = helper.normal_path(dir);
    //校验签名
    if (await helper.sign_receiver(fields, "", mch_key) != sign) {
        return ctx.error("错误的签名");
    }
    if (bak && dir && fs.existsSync(dir)) {
        await exebakdir_tozip(dir, bak);
        return ctx.success();
    } else {
        return ctx.error(`备份失败！bak:${bak},dir:${dir},目标目录是否存在:${fs.existsSync(bak)},源目录是否存在：${fs.existsSync(dir)}`);
    }
})


async function exec_cmd(path, cmd) {
    return new Promise((resolve, reject) => {
        exec(cmd, { cwd: path }, function (err, stdout, stderr) {
            console.log(stdout);
            if (err) {
                reject(err);
            } else {
                resolve(stdout);
            }
        });
    });
}


router.post("/exec_cmd", async function (ctx) {
    var data = ctx.request.body;
    let { sign, path, cmd } = data;
    delete data.sign;
    let mysign = await helper.sign_receiver(data, "", mch_key);
    if (mysign != sign) {
        return ctx.error("签名错误");
    }
    var rs = await exec_cmd(path, cmd);
    console.log(path, cmd);
    return ctx.success({}, { message: "cmd:" + cmd + ",path:" + path + "执行结果:" + rs });
})






router.all("/adc", function (ctx) {
    ctx.body = "adc";
})

app.use(router.routes());

app.context.onerror = function (ex) {
    if (!ex) return;
    this.res.headerSent = true;
    this.res.end(JSON.stringify({ result: false, message: ex.message, err_code: "500" }))
    console.log(ex);
    return;
}

app.listen(port);


function mkdirs(dirpath, mode) {
    var exists = fs.existsSync(dirpath);
    if (exists) return;
    //尝试创建父目录，然后再创建当前目录
    mkdirs(path.dirname(dirpath), mode);
    fs.mkdirSync(dirpath, mode);

};

//备份目录到另一个目录
async function exebakdir_tozip(fromdir, todir) {
    var archive = archiver('zip', {
        zlib: { level: 9 } // Sets the compression level.
    });
    mkdirs(todir, 0777);
    return new Promise((resolve, reject) => {

        var name = path.relative(path.join(fromdir, "../"), fromdir);
        var output = fs.createWriteStream(todir + '/' + name + new Date().format("yyyy_MM_dd_HHmmss") + ".zip");
        output.on("end", function () {
            resolve(true);
        })
        output.on('close', function () {
            resolve(true);
        });

        output.on("error", function (err) {
            reject(err);
        })
        archive.pipe(output);
        //-------------------new-----------------------------
        let dirrs = fs.readdirSync(fromdir);
        dirrs.forEach((item, i) => {
            if (item == "node_modules" || item == "logs") return true;
            let abspath = path.join(fromdir, item);
            console.log(abspath);
            if (fs.lstatSync(abspath).isDirectory()) {
                archive.directory(abspath, item);
            } else {
                archive.file(abspath, { name: item })
            }


        })
        //-------------------------------------------------------
        // archive.directory(fromdir, false);
        archive.finalize();
    });



}
function copyfile(from, to) {
    var exists = fs.existsSync(to);
    if (exists) {
        console.log("文件存在，删除： " + to);
        fs.unlinkSync(to);
        var content = fs.readFileSync(from);
        fs.writeFileSync(to, content);

    } else {
        var exists = fs.existsSync(path.dirname(to));
        if (exists) {
            console.log("has path " + path.dirname(to))
            var content = fs.readFileSync(from);
            fs.writeFileSync(to, content);

        } else {
            console.log("no has path " + path.dirname(to))
            mkdirs(path.dirname(to), 0777);
            var content = fs.readFileSync(from);
            fs.writeFileSync(to, content);

        }
    }
}